@@ -7,7 +7,11 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <afsconfig.h>
 #include <afs/param.h>
+
+RCSID("$Header: /cvs/openafs/src/butm/file_tm.c,v 1.5.2.1 2003/07/29 22:36:10 shadow Exp $");
+
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #else
@@ -26,6 +30,15 @@
 #include <afs/usd.h>
 #include "error_macros.h"
 
+#ifdef O_LARGEFILE
+typedef off64_t         osi_lloff_t;
+#else /* O_LARGEFILE */
+#ifdef AFS_HAVE_LLSEEK
+typedef offset_t        osi_lloff_t;
+#else /* AFS_HAVE_LLSEEK */
+typedef off_t           osi_lloff_t;
+#endif /* AFS_HAVE_LLSEEK */
+#endif /* O_LARGEFILE */
  
 extern int isafile;
 
@@ -71,7 +84,6 @@ extern int isafile;
  * in succession (2 or 4 of them ).
  */
 
-
 struct fileMark {			/* in network byte order */
     afs_int32 magic;
     afs_uint32 nBytes;
@@ -666,7 +678,7 @@ incPosition(info, fid, dataSize)
    if ( info->posCount >=  2147467264 )                 /* 2GB - 16K */
    {
       info->posCount = 0;
-#if (defined(AFS_SUN_ENV) || defined(AFS_DEC_ENV))
+#if (defined(AFS_SUN_ENV) || defined(AFS_DEC_ENV) || defined(AFS_LINUX24_ENV))
       if (!isafile)
 	{
 	  hset64(off, 0, 0);
@@ -804,7 +816,7 @@ WriteTapeBlock(info, buffer, length, blockType)
    {
        if (length == 0) ERROR_EXIT(0);
        bmark = (struct blockMark *)buffer;
-       bzero(bmark, sizeof(struct blockMark));
+       memset(bmark, 0, sizeof(struct blockMark));
        bmark->magic = htonl(BLOCK_MAGIC);
        bmark->count = htonl(length);
    }
@@ -895,7 +907,7 @@ ReadTapeBlock(info, buffer, blockType)
 
    p = (struct progress *)info->tmRock;
 
-   bzero(buffer, BUTM_BLOCKSIZE);
+   memset(buffer, 0, BUTM_BLOCKSIZE);
    label = (struct tapeLabel *)buffer;
    fmark = (struct fileMark  *)buffer;
    bmark = (struct blockMark *)buffer;
@@ -1205,12 +1217,12 @@ static afs_int32 file_WriteLabel (info, label, rewind)
 
     /* Copy the label into the tape block
      * ---------------------------------- */
-    bzero(tapeBlock, BUTM_BLOCKSIZE);
+    memset(tapeBlock, 0, BUTM_BLOCKSIZE);
 
     if (!label->creationTime) label->creationTime = time(0);
 
     tlabel = (struct tapeLabel *)tapeBlock;
-    bcopy(label, &tlabel->label, sizeof(struct butm_tapeLabel));
+    memcpy(&tlabel->label, label, sizeof(struct butm_tapeLabel));
     tlabel->label.structVersion  = htonl(CUR_TAPE_VERSION);
     tlabel->label.creationTime   = htonl(tlabel->label.creationTime);
     tlabel->label.expirationDate = htonl(tlabel->label.expirationDate);
@@ -1308,7 +1320,7 @@ file_ReadLabel (info, label, rewind)
     if (label) 
     {
         tlabel = (struct tapeLabel *) tapeBlock;
-	bcopy(&tlabel->label, label, sizeof(struct butm_tapeLabel));
+	memcpy(label, &tlabel->label, sizeof(struct butm_tapeLabel));
 	label->structVersion  = ntohl(label->structVersion);
 	label->creationTime   = ntohl(label->creationTime);
 	label->expirationDate = ntohl(label->expirationDate);
@@ -1409,7 +1421,7 @@ file_WriteFileData (info, data, blocks, len)
 
         if (len < BUTM_BLKSIZE)
 	{
-	    bzero(&dstart[len], BUTM_BLKSIZE - len);
+	    memset(&dstart[len], 0, BUTM_BLKSIZE - len);
 	    length = len;
 	}
 	else
@@ -1426,7 +1438,7 @@ file_WriteFileData (info, data, blocks, len)
 
 	if (b < (blocks-1)) b++;
 	else if (len) 
-	    bcopy(&dstart[BUTM_BLKSIZE], &dstart[0], len);
+	    memcpy(&dstart[0], &dstart[BUTM_BLKSIZE], len);
     }
 
 error_exit:
@@ -1570,7 +1582,9 @@ static afs_int32 file_Seek (info, position)
     afs_int32 position;
 {   
     afs_int32 code = 0;
-    afs_int32 posit, w;
+    afs_int32 w;
+    osi_lloff_t posit;
+    afs_uint32 c, d;
     struct progress *p;
     afs_hyper_t startOff, stopOff;  /* for normal file(non-tape)  seeks  */
 
@@ -1584,9 +1598,17 @@ static afs_int32 file_Seek (info, position)
     if (isafile)
     {
 	p = (struct progress *)info->tmRock;
-        posit = (position * BUTM_BLOCKSIZE);
-	
-	hset64(startOff, 0, posit);
+        posit = (osi_lloff_t)position * (osi_lloff_t)BUTM_BLOCKSIZE;
+	/* Not really necessary to do it this way, should be fixed. */
+#ifdef O_LARGEFILE
+	c = (posit >> 32);
+	d = (posit & 0xffffffff);
+#else
+	c = 0;
+	d = posit;
+#endif
+	hset64(startOff, c, d);
+
 	w = USD_SEEK(p->fid, startOff, SEEK_SET, &stopOff);
 	if (w) info->error == w;
 	if (hcmp(startOff, stopOff) != 0) ERROR_EXIT(BUTM_POSITION);
@@ -1741,7 +1763,7 @@ afs_int32 butm_file_Instantiate (info, file)
     if (!info)                                    ERROR_EXIT(BUTM_BADARGUMENT);
     if (info->structVersion != BUTM_MAJORVERSION) ERROR_EXIT(BUTM_OLDINTERFACE);
 
-    bzero (info, sizeof(struct butm_tapeInfo));
+    memset(info, 0, sizeof(struct butm_tapeInfo));
     info->structVersion      = BUTM_MAJORVERSION;
     info->ops.mount          = file_Mount;
     info->ops.dismount       = file_Dismount;
